#!/usr/bin/python

# by Chris 'fydo' Hopp for microLD1

import pygame, os, sys, optparse

libdir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'lib'))
sys.path.insert(0, libdir)

from pygame.locals import *
from game import *
from outro import *

USAGE = """%prog [-fn]"""
systemVersion = "0.1"
VERSION = "%prog v" + systemVersion


def parse_options():
	"""parse_options() -> opts, args

	Parse any command-line options given.
	Returns both the parsed options and arguments.
	"""

	parser = optparse.OptionParser(usage=USAGE, version=VERSION)

	parser.add_option("-w", "--windowed",
			action="store_true", default=False, dest="windowed",
			help="enable windowed mode")

	parser.add_option("-n", "--nosound",
			action="store_true", default=False, dest="nosound",
			help="disable sound")
	
	opts, args = parser.parse_args()
	return opts, args

def intro(screen, nosound):

	intrographic = pygame.image.load(os.path.join('data', 'title-white.png')).convert()
	screen.fill((255,255,255))
	pygame.display.flip()
	pygame.time.wait(500)

	wavesound = pygame.mixer.Sound(os.path.join('data', 'waves.ogg'))
	if not nosound:
		wavesound.play()
	
	#fade in 1
	for x in range(450):
		for event in pygame.event.get():
			if event.type == pygame.QUIT: 
				sys.exit()
		screen.fill((255,255,255))
		work = intrographic
		#alpha = ((x * 255) / 40.0)
		work.set_alpha((x * 255) / 450.0, RLEACCEL)
		screen.blit(work, (0,0))
		pygame.display.flip()
		pygame.time.wait(10)
	
	pygame.time.wait(1000)
	
	if not nosound:
		wavesound.play()
	
	# fade to blacks
	for x in range(450):
		for event in pygame.event.get():
			if event.type == pygame.QUIT: 
				sys.exit()
		temp = x
		x = 450 - x
		screen.fill((0,0,0))
		work = intrographic
		work.set_alpha((x * 255) / 450.0, RLEACCEL)
		screen.blit(work, (0,0))
		pygame.display.flip()
		pygame.time.wait(10)
		
	pygame.time.wait(800)
	
def main(windowed, nosound):
	try:
		import psyco
		psyco.background()
		#print 'psyco found and enabled!'
	except ImportError:
		pass
		#print 'psyco not found! don\'t worry, this game doesn\'t really need it.'

	pygame.mixer.pre_init(22050, -16, 1, 2048)
	pygame.init()
	
	size = width, height = 320, 200
	if not windowed:
		screen = pygame.display.set_mode(size, FULLSCREEN)
		#pygame.time.wait(200)#wait for the computer to switch video modes
	else:
		screen = pygame.display.set_mode(size, HWSURFACE|DOUBLEBUF)
		#pygame.time.wait(200)
	
	pygame.display.set_caption('paradise gone v' + systemVersion )
	#pygame.mouse.set_visible(0)
	
	intro(screen, nosound)

	game = Game(nosound)
	result = game.Run(screen)

	if result:
		outro = Outro(nosound)
		outro.Run(screen)
	
	pygame.display.quit()
	return


def Start():
	# Welcome the player
	print 'paradise gone v' + systemVersion + ' - http://www.fydo.net'

	#Parse the options up! Woo
	opts, args = parse_options()

	# Play time
	main(opts.windowed, opts.nosound)
	
	# Say goodbye like every good little program should
	print 'thanks for playing!'
		
		
if __name__ == '__main__':
	Start()



