#!/usr/bin/python

# by Chris 'fydo' Hopp for microLD1

import os, sys, math, random, pygame
from pygame.locals import *

LogicalFPS = 64.0
TicksPerFrame = (1000.0 / LogicalFPS)
MaxFrameSkip = 2
SPEED = 75.0
LETTERSPEED = 0.04
TILESIZE = 32
PANSPEED = 70.0

class Camera:
	def __init__(self):
		self.pos = (0,0)
		
	def GetPos(self):
		return self.pos
		
	def SetPos(self, newpos):
		self.pos = newpos

class Player:
	def __init__(self, pos):
		self.pos = pos
		self.mappos = (int(self.pos[0] / 32.0), int(self.pos[1] / 32.0))
		self.gfx = pygame.image.load(os.path.join('data', 'player.png')).convert_alpha()
		self.moving = False
		# 0 = up
		# 1 = down
		# 2 = left
		# 3 = right
		self.direction = 1
		self.frame = 0
		self.frametimer = 0
		self.travelled = 0
		
	def MoveUp(self):
		if not self.moving:
			self.moving = True
			self.direction = 0
	
	def MoveDown(self):
		if not self.moving:
			self.moving = True
			self.direction = 1
		
	def MoveLeft(self):
		if not self.moving:
			self.moving = True
			self.direction = 2
		
	def MoveRight(self):
		if not self.moving:
			self.moving = True
			self.direction = 3
	
	def Compute(self, dt, camera, nocamera):
		if self.moving == True:
		
			done = False
			dist = dt * SPEED
			oldtravel = self.travelled
			self.travelled += dist
		
			if self.travelled > TILESIZE:
				dist = TILESIZE - oldtravel
				done = True
		
			if self.direction == 0:
				if (self.pos[1] == 0):
					done = True
				else:
					self.pos = self.pos[0], self.pos[1] - dist
			elif self.direction == 1:
				if (self.pos[1] == 352):
					done = True
				else:
					self.pos = self.pos[0], self.pos[1] + dist
			elif self.direction == 2:
				if (self.pos[0] == 0):
					done = True
				else:
					self.pos = self.pos[0] - dist, self.pos[1]
			elif self.direction == 3:
				if (self.pos[0] == 608):
					done = True
				else:
					self.pos = self.pos[0] + dist, self.pos[1]
				
			if done == True:
				self.mappos = (int(self.pos[0] / 32.0), int(self.pos[1] / 32.0))
				self.travelled = 0
				self.moving = False
				
			self.frametimer += dt
			
			if self.frametimer > 0.11:
				self.frametimer = 0
				self.frame += 1
				if self.frame == 4:
					self.frame = 0
			
		else:
			self.frame = 0
			self.frametimer = 0
		
		if not nocamera:
			if (self.pos[0] + TILESIZE) > (camera.GetPos()[0] + 288) and (self.pos[0] + TILESIZE) < 608:
				camera.SetPos((self.pos[0] + TILESIZE - 288, camera.GetPos()[1]))
			if (self.pos[0]) < camera.GetPos()[0] + 32:
				camera.SetPos((self.pos[0] - 32, camera.GetPos()[1]))
			if (self.pos[1] + TILESIZE) > (camera.GetPos()[1] + 168) and (self.pos[1] + TILESIZE) < 368:
				camera.SetPos((camera.GetPos()[0], self.pos[1] + TILESIZE - 168))
			if (self.pos[1]) < camera.GetPos()[1] + 32:
				camera.SetPos((camera.GetPos()[0], self.pos[1] - 32))
				
	def Draw(self, camera, screen):
		#screen.blit(self.launcher.image, (self.launcher.rect), ((self.launcher.curFrame * 200), 0, 200, 150))
		
		newpos = (self.pos[0] - camera[0], self.pos[1] - camera[1])
		screen.blit(self.gfx, (newpos), (self.frame * TILESIZE, self.direction * TILESIZE, TILESIZE, TILESIZE))
		
class Game:
	def __init__(self, noSound):
		self.noSound = noSound
		self.musicStart = False
		self.done = False
		self.timerdiff = 0
		self.ShowIntro = True
		self.IntroTimer = 0.00
		self.levelBG = pygame.image.load(os.path.join('data', 'level.png')).convert()
		self.buttongfx = pygame.image.load(os.path.join('data', 'button.png')).convert_alpha()
		self.objectgfx = pygame.image.load(os.path.join('data', 'objects.png')).convert_alpha()
		self.wavesound = pygame.mixer.Sound(os.path.join('data', 'waves.ogg'))
		pygame.mixer.music.load(os.path.join('data', 'apockalips.ogg'))
		pygame.mixer.music.set_volume(0.6)
		self.itemsfound = 0
		self.Message = ''
		self.AllowInput = False
		self.TargetMessage = "You awake to find yourself on a beach."
		self.panningUp = False
		self.panningDown = False
		self.triggeredmessage = 0
		self.currdialog = -1
		self.lettercount = 0
		self.lettertimer = 0.00
		self.WaitForInput = False
		self.WaitForInputTimer = 0.00
		self.WaitForInputFrame = False
		self.player = Player((128,128))
		self.camera = Camera()
		self.font = pygame.font.Font(None, 14)
		
		#dialog data
		self.dialog = ['It\'s oddly familiar. You\'ve been here before.', 'x',
						'That\'s right... You own a beach house near here? Maybe?', 'x',
						'What?! What happened!?', 'y', 
						'The beach house ... lost in a fire ...', 'd', 
						'All you have left are the memories of what you\'ve lost ...', 'I\'d like to recount those memories. Press \'z\' near objects.', 'x',
						'It\'s what\'s left of your favorite shirt ...', 'x',
						'You had some good times with these vinyl records ...', 'x',
						'Your childhood teddy bear, scorched by flame ...', 'x',
						'Your prized bowling trophy. It\'s seen better days.', 'x',
						'A photograph of you and friends. It\'s burnt.', 'x',
						'You decide to check out your surfboard later.', 'x',
						'You look at your favorite surfboard.', 
						'You decide to make the most of your dismal dilemma.', 'e', 'x']
		
		#level data
		self.leveldata = 	[[9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9],
							 [9,9,9,0,0,9,9,9,9,9,9,0,3,0,0,5,9,9,9,9],
							 [9,9,9,0,0,0,9,9,9,9,9,0,0,0,0,0,0,9,9,9],
						     [9,9,9,0,0,0,9,9,9,9,0,0,0,6,0,0,4,9,9,9],
							 [9,9,9,0,0,0,0,9,9,9,0,7,0,0,0,0,0,0,9,9],
							 [9,9,9,0,0,0,0,9,9,9,0,0,0,0,0,0,0,0,8,9],
							 [9,9,9,0,0,0,0,9,9,9,9,9,9,0,0,0,0,0,0,0],
							 [9,9,9,9,0,0,0,9,9,9,9,9,9,9,9,0,0,0,0,0],
							 [9,9,9,9,9,0,0,1,0,9,9,9,9,9,9,9,2,2,2,2],
							 [9,9,9,9,9,0,1,0,0,0,0,0,0,0,0,0,0,0,0,9],
							 [9,9,9,9,9,9,0,0,0,0,0,0,0,0,0,0,0,0,0,9],
							 [9,9,9,9,9,9,9,0,0,0,0,0,0,0,0,0,0,0,9,9]]
							 
	def FadeOut(self, screen):
	
		self.done = True
	
		if not self.noSound:
			pygame.mixer.music.fadeout(300)
			self.wavesound.play()
	
		for x in range(300):
			for event in pygame.event.get():
				if event.type == pygame.QUIT: 
					sys.exit()
			temp = x
			x = 300 - x
			screen.fill((0,0,0))
			work = self.levelBG
			work.set_alpha((x * 255) / 300.0, RLEACCEL)
			screen.blit(work, (-self.camera.GetPos()[0], -self.camera.GetPos()[1]))
			self.player.Draw(self.camera.GetPos(), screen)
			pygame.display.flip()
			pygame.time.wait(10)
	
			
	def FadeIn(self, screen):
		if not self.noSound:
			self.wavesound.play()
	
		for x in range(250):
			for event in pygame.event.get():
				if event.type == pygame.QUIT: 
					sys.exit()
			screen.fill((0,0,0))
			work = self.levelBG
			work.set_alpha((x * 255) / 250.0, RLEACCEL)
			screen.blit(work, (0,0))
			self.player.Draw(self.camera.GetPos(), screen)
			pygame.display.flip()
			pygame.time.wait(10)
	
	def AbleToMove(self, coordx, coordy):
		if (coordx < 0) or (coordy < 0):
			return False
		elif (coordx > 19) or (coordy > 11):
			return False
			
		if self.leveldata[coordy][coordx] >= 3:
			return False
		else:
			return True
	
	def CheckForItem(self):
		#check in all 4 directions
		found = False
		num = 0
		thedialog = 0
		
		if self.player.mappos[0] < 19:
			if (self.leveldata[self.player.mappos[1]][self.player.mappos[0] + 1] in (3,4,5,6,7,8)):
				found = True
				num = self.leveldata[self.player.mappos[1]][self.player.mappos[0] + 1]
				if num != 8:
					self.leveldata[self.player.mappos[1]][self.player.mappos[0] + 1] = 0
		if self.player.mappos[0] > 1:
			if (self.leveldata[self.player.mappos[1]][self.player.mappos[0] - 1] in (3,4,5,6,7,8)):
				found = True
				num = self.leveldata[self.player.mappos[1]][self.player.mappos[0] - 1]
				if num != 8:
					self.leveldata[self.player.mappos[1]][self.player.mappos[0] - 1] = 0
		if self.player.mappos[1] < 10:
			if (self.leveldata[self.player.mappos[1] + 1][self.player.mappos[0]] in (3,4,5,6,7,8)):
				found = True
				num = self.leveldata[self.player.mappos[1] + 1][self.player.mappos[0]]
				if num != 8:
					self.leveldata[self.player.mappos[1] + 1][self.player.mappos[0]] = 0
		if self.player.mappos[1] > 1:
			if (self.leveldata[self.player.mappos[1] - 1][self.player.mappos[0]] in (3,4,5,6,7,8)):
				found = True
				num = self.leveldata[self.player.mappos[1] - 1][self.player.mappos[0]]
				if num != 8:
					self.leveldata[self.player.mappos[1] - 1][self.player.mappos[0]] = 0
			
		if found:
			if num == 3:
				thedialog = 11
			elif num == 4:
				thedialog = 13
			elif num == 5:
				thedialog = 15
			elif num == 6:
				thedialog = 17
			elif num == 7:
				thedialog = 19
			elif num == 8:
				thedialog = 21
			
			self.currdialog = thedialog
			self.TargetMessage = self.dialog[thedialog]
			
			if num != 8:
				self.itemsfound += 1
			
			if num == 8:
				if self.itemsfound == 5:
					self.currdialog = 23
					self.TargetMessage = self.dialog[23]
			
	
	def DoInput(self, screen):
		for event in pygame.event.get():
			if event.type == pygame.QUIT: 
				sys.exit()
			elif event.type == KEYDOWN and self.AllowInput:
				if (event.key == K_ESCAPE):	
					#self.music.fadeout(500)
					return False
				elif (event.key == K_UP) and self.WaitForInput == False:
					if self.AbleToMove(self.player.mappos[0], self.player.mappos[1] - 1):
						self.player.MoveUp()
				elif (event.key == K_DOWN) and self.WaitForInput == False:
					if self.AbleToMove(self.player.mappos[0], self.player.mappos[1] + 1):
						self.player.MoveDown()
				elif (event.key == K_LEFT) and self.WaitForInput == False:
					if self.AbleToMove(self.player.mappos[0] - 1, self.player.mappos[1]):
						self.player.MoveLeft()
				elif (event.key == K_RIGHT) and self.WaitForInput == False:	
					if self.AbleToMove(self.player.mappos[0] + 1, self.player.mappos[1]):
						self.player.MoveRight()
				elif (event.key == K_z) and self.WaitForInput:
					self.WaitForInput = False
					self.currdialog += 1
					self.Message = ''
					
					if self.dialog[self.currdialog] == 'x':
						if not self.musicStart and not self.noSound:
							pygame.mixer.music.play(-1)
							self.musicStart = True
						self.TargetMessage = ''
					elif self.dialog[self.currdialog] == 'y':
						self.TargetMessage = ''
						self.AllowInput = False
						self.panningUp = True
					elif self.dialog[self.currdialog] == 'd':
						self.TargetMessage = ''
						self.AllowInput = False
						self.panningDown = True
					elif self.dialog[self.currdialog] == 'e':
						#end!
						self.FadeOut(screen)
					else:
						self.TargetMessage = self.dialog[self.currdialog]
						self.AllowInput = False
					
				elif (event.key == K_z) and self.WaitForInput == False:
					#pass
					#select object
					self.CheckForItem()
					
				#else:
					#self.ButtonDown()
					
			#elif event.type == KEYUP:
			#	self.ButtonUp()
			#elif (event.type == MOUSEBUTTONDOWN) or (event.type == JOYBUTTONDOWN):
			#	self.ButtonDown()
			#elif (event.type == MOUSEBUTTONUP) or (event.type == JOYBUTTONUP):
			#	self.ButtonUp()
			
		return True

	def Compute(self, timestep):
		if self.TargetMessage != self.Message:
			self.AllowInput = False
			self.lettertimer += timestep
			
			if self.lettertimer >= LETTERSPEED:
				self.lettertimer = 0
				self.lettercount += 1
				
				self.Message = self.TargetMessage[:self.lettercount]
				
				if self.TargetMessage == self.Message:
					self.lettercount = 0
					self.AllowInput = True
					self.WaitForInput = True
								
		if self.WaitForInput:
			self.WaitForInputTimer += timestep
			
			if self.WaitForInputTimer > 0.13:
				self.WaitForInputFrame = not self.WaitForInputFrame
				self.WaitForInputTimer = 0.0

		#panning
		if (self.panningDown):
			self.camera.SetPos((self.camera.GetPos()[0], self.camera.GetPos()[1] + (timestep * PANSPEED)))
			
			if self.camera.GetPos()[1] > 128:
				self.camera.SetPos((self.camera.GetPos()[0], 128))
				self.panningDown = False
				self.panningUp = False
				self.currdialog = 8
				self.TargetMessage = self.dialog[8]
				self.AllowInput = True
		elif (self.panningUp):
			self.camera.SetPos((self.camera.GetPos()[0] + (timestep * PANSPEED), self.camera.GetPos()[1] - (timestep * PANSPEED)))
			
			if self.camera.GetPos()[0] > 320:
				self.camera.SetPos((320, (self.camera.GetPos()[1])))
			
			if self.camera.GetPos()[1] < 0:
				self.camera.SetPos((self.camera.GetPos()[0], 0))
				self.currdialog = 6
				self.TargetMessage = self.dialog[6]
				self.AllowInput = True
		
				
		#check for messages
		if self.leveldata[self.player.mappos[1]][self.player.mappos[0]] == 1:
			if self.triggeredmessage < 1:
				self.triggeredmessage = 1
				self.currdialog = 2
				self.TargetMessage = self.dialog[2]
		elif self.leveldata[self.player.mappos[1]][self.player.mappos[0]] == 2:
			if self.triggeredmessage < 2:
				self.triggeredmessage = 2
				self.currdialog = 4
				self.TargetMessage = self.dialog[4]
		
	def drawStatus(self, screen, statustext):
		statusBar = pygame.Surface((320, 16)).convert()
		statusBar.fill((20,20,20))
		screen.blit(statusBar, (0, 184))
	
		text = self.font.render(statustext, 1, (255,255,255))
		textpos = text.get_rect(centerx = screen.get_width()/2, centery = ((text.get_height()/2)))
		screen.blit(text, (textpos[0], 188 + textpos[1]))

	def Run(self, screen):
		LastTicks = 0
		ElapsedTicks = 0

		self.FadeIn(screen)
		
		if (not self.noSound):
			pass
			#pygame.mixer.music.play(-1)

		while not self.done:
			self.thetime = pygame.time.get_ticks()
			ElapsedTicks += self.thetime - LastTicks
			LastTicks = self.thetime
			
			if (ElapsedTicks > (TicksPerFrame * 4)):
				ElapsedTicks = 0

			frames = 0

			if (ElapsedTicks > TicksPerFrame):
				while ((ElapsedTicks > TicksPerFrame) and (frames < MaxFrameSkip)):
					ElapsedTicks -= TicksPerFrame
					timestep = 1.0 / LogicalFPS

					if (self.DoInput(screen) == False):
						pygame.mixer.music.stop()
						return False

					if self.AllowInput:
						self.player.Compute(timestep, self.camera, self.panningUp or self.panningDown)
					
					self.Compute(timestep)
						
					frames += 1

			# draw everything!
			if not self.done:
				screen.blit(self.levelBG, (-self.camera.GetPos()[0], -self.camera.GetPos()[1]))
			
				for x in range(19):
					for y in range(11):
						if self.leveldata[y][x] == 3:
							screen.blit(self.objectgfx, ((x * 32) - self.camera.GetPos()[0],(y * 32) - self.camera.GetPos()[1]), (0,0,32,32))
						elif self.leveldata[y][x] == 4:
							screen.blit(self.objectgfx, ((x * 32) - self.camera.GetPos()[0],(y * 32) - self.camera.GetPos()[1]), (32,0,32,32))
						elif self.leveldata[y][x] == 5:
							screen.blit(self.objectgfx, ((x * 32) - self.camera.GetPos()[0],(y * 32) - self.camera.GetPos()[1]), (64,0,32,32))
						elif self.leveldata[y][x] == 6:
							screen.blit(self.objectgfx, ((x * 32) - self.camera.GetPos()[0],(y * 32) - self.camera.GetPos()[1]), (96,0,32,32))
						elif self.leveldata[y][x] == 7:
							screen.blit(self.objectgfx, ((x * 32) - self.camera.GetPos()[0],(y * 32) - self.camera.GetPos()[1]), (128,0,32,32))
						elif self.leveldata[y][x] == 8:
							screen.blit(self.objectgfx, ((x * 32) - self.camera.GetPos()[0],(y * 32) - self.camera.GetPos()[1]), (160,0,32,32))
				
			
				self.player.Draw(self.camera.GetPos(), screen)
				#text = self.font.render('' + str(self.player.mappos[0]) + ', ' + str(self.player.mappos[1]) + ' -- ' + str(self.camera.GetPos()[0]) + ', ' + str(self.camera.GetPos()[1]), 1, (255,255,255), (0,0,0))
				#screen.blit(text, (0,0))
				
				
				if len(self.Message):
					self.drawStatus(screen, self.Message)
				
				if (self.WaitForInput):
					xplace = 0
					
					if self.WaitForInputFrame:
						xplace = 11
						
					screen.blit(self.buttongfx, (307, 187), (xplace, 0, 11, 11))
				
			if self.timerdiff == 0:
				timer = pygame.time.get_ticks()

			pygame.display.flip()
			# delay time
			pygame.time.wait(10)
	
			if self.done:
				return True

