#!/usr/bin/python

# by Chris 'fydo' Hopp for microLD1

import os, sys, math, random, pygame
from pygame.locals import *

LogicalFPS = 64.0
TicksPerFrame = (1000.0 / LogicalFPS)
MaxFrameSkip = 2
SPEED = 180.0

class Outro:
	def __init__(self, nosound):
		self.nosound = nosound
		self.timerdiff = 0
		self.BGimage = pygame.image.load(os.path.join('data', 'outrobg.png')).convert()
		self.Credits = pygame.image.load(os.path.join('data', 'credits.png')).convert_alpha()
		self.surfy = pygame.image.load(os.path.join('data', 'surfy.png')).convert_alpha()
		pygame.mixer.music.load(os.path.join('data', 'surftopia2.ogg'))
		pygame.mixer.music.set_volume(0.6)
		self.thetime = 0.00
		self.done = False
		self.playerx = 320
		self.playery = 100
		self.frametimer = 0.0
		self.frame = 0
		self.creditpos = 0.0
		self.moveup = False
		self.movedown = False
	
	def FadeIn(self, screen):
		for x in range(50):
			screen.fill((0,0,0))
			work = self.BGimage
			work.set_alpha((x * 255) / 50.0, RLEACCEL)
			screen.blit(work, (0,0))
			pygame.display.flip()
			pygame.time.wait(10)
	
	def DoInput(self, dt):
		for event in pygame.event.get():
			if event.type == pygame.QUIT: 
				sys.exit()
			elif event.type == KEYDOWN:
				if (event.key == K_ESCAPE):	
					return False
				elif (event.key == K_UP):
					self.moveup = True
				elif (event.key == K_DOWN):
					self.movedown = True
			elif event.type == KEYUP:
				if (event.key == K_UP):
					self.moveup = False
				elif (event.key == K_DOWN):
					self.movedown = False
	
	def Compute(self, dt):
		if self.playerx > 205:
			self.playerx -= dt * (SPEED * 0.5)
			if self.playerx < 205:
				self.playerx = 205
				
		if self.moveup:
			self.playery -= dt * SPEED
			if self.playery < 80:
				self.playery = 80
		elif self.movedown:
			self.playery += dt * SPEED
			if self.playery > 168:
				self.playery = 168
				
		self.frametimer += dt
			
		if self.frametimer > 0.1:
			self.frametimer = 0
			self.frame += 1
			if self.frame > 3:
				self.frame = 0
		
		self.creditpos += dt * 17
		
		if self.creditpos > 650:
			self.done = True
	
	def Run(self, screen):
		if not self.nosound:
			pygame.mixer.music.play(-1)
		
		LastTicks = 0
		ElapsedTicks = 0
		
		self.FadeIn(screen)
		
		while not self.done:
			self.thetime = pygame.time.get_ticks()
			ElapsedTicks += self.thetime - LastTicks
			LastTicks = self.thetime
			
			if (ElapsedTicks > (TicksPerFrame * 4)):
				ElapsedTicks = 0

			frames = 0

			if (ElapsedTicks > TicksPerFrame):
				while ((ElapsedTicks > TicksPerFrame) and (frames < MaxFrameSkip)):
					ElapsedTicks -= TicksPerFrame
					timestep = 1.0 / LogicalFPS

					if (self.DoInput(timestep) == False):
						pygame.mixer.music.stop()
						return False

					self.Compute(timestep)
						
					frames += 1
			
			#draws!
			screen.blit(self.BGimage, (0,0))
			
			screen.blit(self.surfy, (self.playerx, self.playery), (self.frame * 32, 0, 32, 32))
			
			screen.blit(self.Credits, (0, 200 - self.creditpos))
			
			if self.timerdiff == 0:
				timer = pygame.time.get_ticks()

			pygame.display.flip()
			# delay time
			pygame.time.wait(10)
			
			
			